@@ -975,7 +975,7 @@ new_window_callback (GtkWidget      *menu_item,
   terminal_app_new_terminal (terminal_app_get (),
                              terminal_profile_get_for_new_term (screen->priv->profile),
                              NULL,
-                             FALSE, FALSE, NULL, NULL, NULL, NULL);
+                             FALSE, FALSE, NULL, NULL, NULL, NULL, NULL);
 }
 
 static void
@@ -985,7 +985,7 @@ new_tab_callback (GtkWidget      *menu_item,
   terminal_app_new_terminal (terminal_app_get (),
                              terminal_profile_get_for_new_term (screen->priv->profile),
                              screen->priv->window,
-                             FALSE, FALSE, NULL, NULL, NULL, NULL);
+                             FALSE, FALSE, NULL, NULL, NULL, NULL, NULL);
 }
 
 static void
@@ -1702,14 +1702,15 @@ drag_data_received  (GtkWidget *widget, GdkDragContext *context,
         
         str = gtk_selection_data_get_text (selection_data);
 
-        /*
-	 * pass UTF-8 to the terminal widget. The terminal widget
-	 * should know which encoding mode it's in and be able
-	 * to perform the correct conversion.
+        /* FIXME should this be converted back to locale encoding
+         * before sending to the terminal? Most likely yes.
          */
         if (str && *str)
-	  terminal_widget_write_data_to_child (screen->priv->term,
-					       str, strlen (str));
+          {
+            terminal_widget_write_data_to_child (screen->priv->term,
+                                                 str, strlen (str));
+          }
+
         g_free (str);
       }
       break;
@@ -2143,6 +2144,8 @@ make_font_monospace (const PangoFontDescription *font)
 {
   PangoFontDescription *ret;
 
+  return NULL;
+  
   ret = pango_font_description_copy (font);
 
   /* FIXME */
