@@ -143,6 +143,17 @@ void terminal_widget_write_data_to_child (GtkWidget  *widget,
                                           const char *data,
                                           int         len);
 
+void terminal_widget_set_pango_font (GtkWidget                  *widget,
+                                     const PangoFontDescription *font_desc);
+
+gboolean terminal_widget_supports_pango_fonts (void);
+
+const char* terminal_widget_get_encoding (GtkWidget  *widget);
+void        terminal_widget_set_encoding (GtkWidget  *widget,
+                                          const char *encoding);
+
+gboolean terminal_widget_supports_dynamic_encoding (void);
+
 G_END_DECLS
 
 #endif /* TERMINAL_WIDGET_H */
