@@ -24,6 +24,7 @@
 #include "terminal-widget.h"
 #include "terminal-window.h"
 #include "terminal.h"
+#include "encoding.h"
 #include <string.h>
 #include <stdlib.h>
 #include <libgnome/gnome-program.h>
@@ -57,6 +58,7 @@ struct _TerminalWindowPrivate
   GtkWidget *next_tab_menuitem;
   GtkWidget *previous_tab_menuitem;
   GtkWidget *go_menu;
+  GtkWidget *encoding_menuitem;
   GList *tab_menuitems;
   GList *terms;
   TerminalScreen *active_term;
@@ -145,6 +147,10 @@ static void fullscreen_callback           (GtkWidget      *menuitem,
                                            TerminalWindow *window);
 static void set_title_callback            (GtkWidget      *menuitem,
                                            TerminalWindow *window);
+static void change_encoding_callback      (GtkWidget      *menu_item,
+                                           TerminalWindow *window);
+static void add_encoding_callback         (GtkWidget      *menu_item,
+                                           TerminalWindow *window);
 static void reset_callback                (GtkWidget      *menuitem,
                                            TerminalWindow *window);
 static void reset_and_clear_callback      (GtkWidget      *menuitem,
@@ -164,6 +170,9 @@ static void set_menuitem_text (GtkWidget  *mi,
                                const char *text,
                                gboolean    strip_mnemonic);
 
+static void terminal_menu_opened_callback (GtkWidget      *menuitem,
+                                           TerminalWindow *window);
+
 static gpointer parent_class;
 
 GType
@@ -478,7 +487,120 @@ fill_in_new_term_submenus (TerminalWindow *window)
       tmp = tmp->next;
     }
 
-  g_list_free (profiles);  
+  g_list_free (profiles);
+}
+
+static void
+update_active_encoding_name (TerminalWindow *window)
+{
+  GtkWidget *w;
+  const char *charset;
+  char *name;
+  
+  w = terminal_screen_get_widget (window->priv->active_term);
+  charset = terminal_widget_get_encoding (w);
+
+  name = terminal_encoding_get_name (charset);
+
+#if 0
+  /* doesn't make sense anymore */
+  gtk_label_set_text (GTK_LABEL (gtk_bin_get_child (GTK_BIN (window->priv->encoding_menuitem))),
+                      name);
+#endif
+  
+  g_free (name);
+}
+
+static void
+fill_in_encoding_menu (TerminalWindow *window)
+{
+  GtkWidget *menu;
+  GtkWidget *menu_item;
+  GSList *group;
+  GSList *encodings;
+  GSList *tmp;
+  const char *charset;
+  GtkWidget *w;
+
+  if (!terminal_widget_supports_dynamic_encoding ())
+    return;
+  
+  if (window->priv->active_term == NULL)
+    {
+      gtk_widget_set_sensitive (window->priv->encoding_menuitem, FALSE);
+      gtk_menu_item_set_submenu (GTK_MENU_ITEM (window->priv->encoding_menuitem),
+                                 NULL);
+      return;
+    }
+  
+  gtk_widget_set_sensitive (window->priv->encoding_menuitem, TRUE);
+
+  menu = gtk_menu_new ();
+  gtk_menu_item_set_submenu (GTK_MENU_ITEM (window->priv->encoding_menuitem),
+                             menu);
+
+  w = terminal_screen_get_widget (window->priv->active_term);
+  charset = terminal_widget_get_encoding (w);
+
+  update_active_encoding_name (window);
+  
+  group = NULL;
+  encodings = terminal_get_active_encodings ();
+  tmp = encodings;
+  while (tmp != NULL)
+    {
+      TerminalEncoding *e;
+      char *name;
+      
+      e = tmp->data;
+
+      name = g_strdup_printf ("%s (%s)", e->name, e->charset);
+      
+      menu_item = gtk_radio_menu_item_new_with_label (group, name);
+
+      g_free (name);
+
+      group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (menu_item));
+      gtk_widget_show (menu_item);
+      gtk_menu_shell_append (GTK_MENU_SHELL (menu),
+                             menu_item);
+      
+      if (strcmp (e->charset, charset) == 0)
+        gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menu_item),
+                                        TRUE);
+      
+      g_signal_connect (G_OBJECT (menu_item),
+                        "toggled",
+                        G_CALLBACK (change_encoding_callback),
+                        window);
+
+      g_object_set_data_full (G_OBJECT (menu_item),
+                              "encoding",
+                              g_strdup (e->charset),
+                              (GDestroyNotify) g_free);
+      
+      tmp = tmp->next;
+    }
+
+  g_slist_foreach (encodings,
+                   (GFunc) terminal_encoding_free,
+                   NULL);
+  g_slist_free (encodings);
+
+  menu_item = gtk_separator_menu_item_new ();
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+  gtk_widget_show (menu_item);
+  
+  append_menuitem (menu, _("_Add or Remove..."), NULL,
+                   G_CALLBACK (add_encoding_callback),
+                   window);
+}
+
+static void
+terminal_menu_opened_callback (GtkWidget      *menuitem,
+                               TerminalWindow *window)
+{
+  fill_in_encoding_menu (window);
 }
 
 static void
@@ -614,6 +736,10 @@ terminal_window_init (TerminalWindow *window)
                         "", NULL,
                         NULL, NULL);
   window->priv->terminal_menuitem = mi;  
+
+  g_signal_connect (G_OBJECT (mi), "activate",
+                    G_CALLBACK (terminal_menu_opened_callback),
+                    window);
   
   menu = gtk_menu_new ();
   gtk_menu_set_accel_group (GTK_MENU (menu),
@@ -627,7 +753,18 @@ terminal_window_init (TerminalWindow *window)
   
   append_menuitem (menu, _("_Set Title..."), ACCEL_PATH_SET_TERMINAL_TITLE,
                    G_CALLBACK (set_title_callback), window);
-  
+
+  if (terminal_widget_supports_dynamic_encoding ())
+    {
+      window->priv->encoding_menuitem =
+        append_menuitem (menu,
+                         _("_Character Coding"), NULL, NULL, NULL);
+    }
+  else
+    {
+      window->priv->encoding_menuitem = NULL;
+    }
+
   append_menuitem (menu, _("_Reset"), ACCEL_PATH_RESET,
                    G_CALLBACK (reset_callback), window);
 
@@ -679,7 +816,7 @@ terminal_window_init (TerminalWindow *window)
                               G_CALLBACK (about_callback), window);
   set_menuitem_text (mi, _("_About"),
                      FALSE);
-
+  
   terminal_window_reread_profile_list (window);
   
   terminal_window_set_menubar_visible (window, TRUE);
@@ -748,6 +885,7 @@ terminal_window_destroy (GtkObject *object)
   window->priv->paste_menuitem = NULL;
   window->priv->edit_config_menuitem = NULL;
   window->priv->choose_config_menuitem = NULL;
+  window->priv->encoding_menuitem = NULL;
   
   GTK_OBJECT_CLASS (parent_class)->destroy (object);  
 }
@@ -1822,7 +1960,7 @@ new_window_callback (GtkWidget      *menuitem,
       terminal_app_new_terminal (terminal_app_get (),
                                  profile,
                                  NULL,
-                                 FALSE, FALSE, NULL, NULL, NULL, NULL);
+                                 FALSE, FALSE, NULL, NULL, NULL, NULL, NULL);
     }
 }
 
@@ -1842,7 +1980,7 @@ new_tab_callback (GtkWidget      *menuitem,
       terminal_app_new_terminal (terminal_app_get (),
                                  profile,
                                  window,
-                                 FALSE, FALSE, NULL, NULL, NULL, NULL);
+                                 FALSE, FALSE, NULL, NULL, NULL, NULL, NULL);
     }
 }
 
@@ -2000,6 +2138,38 @@ set_title_callback (GtkWidget      *menuitem,
                                 GTK_WINDOW (window));
 }
 
+static void
+change_encoding_callback (GtkWidget      *menu_item,
+                          TerminalWindow *window)
+{
+  const char *charset;
+  GtkWidget *widget;
+  
+  if (!gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (menu_item)))
+    return;
+
+  if (window->priv->active_term == NULL)
+    return;
+
+  charset = g_object_get_data (G_OBJECT (menu_item),
+                               "encoding");
+
+  g_assert (charset);
+
+  widget = terminal_screen_get_widget (window->priv->active_term);
+  terminal_widget_set_encoding (widget, charset);
+
+  update_active_encoding_name (window);
+}
+
+static void
+add_encoding_callback (GtkWidget      *menu_item,
+                       TerminalWindow *window)
+{
+  terminal_app_edit_encodings (terminal_app_get (),
+                               GTK_WINDOW (window));
+}
+
 static void
 reset_callback (GtkWidget      *menuitem,
                 TerminalWindow *window)
