@@ -1,6 +1,8 @@
 /* terminal program */
 /*
- * Copyright (C) 2001 Havoc Pennington
+ * Copyright (C) 2001, 2002 Havoc Pennington
+ * Copyright (C) 2002 Red Hat, Inc.
+ * Copyright (C) 2002 Sun Microsystems
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,6 +25,7 @@
 #include "terminal-accels.h"
 #include "terminal-window.h"
 #include "profile-editor.h"
+#include "encoding.h"
 #include <gconf/gconf-client.h>
 #include <bonobo-activation/bonobo-activation-activate.h>
 #include <bonobo-activation/bonobo-activation-register.h>
@@ -35,6 +38,7 @@
 #include <popt.h>
 #include <string.h>
 #include <stdlib.h>
+#include <time.h>
 #include <gdk/gdkx.h>
 
 
@@ -64,6 +68,7 @@ struct _TerminalApp
 {
   GList *windows;
   GtkWidget *edit_keys_dialog;
+  GtkWidget *edit_encodings_dialog;
   GtkWidget *new_profile_dialog;
   GtkWidget *new_profile_name_entry;
   GtkWidget *new_profile_base_menu;
@@ -137,6 +142,7 @@ enum {
   OPTION_TAB_WITH_PROFILE,
   OPTION_WINDOW_WITH_PROFILE_ID,
   OPTION_TAB_WITH_PROFILE_ID,
+  OPTION_ROLE,
   OPTION_SHOW_MENUBAR,
   OPTION_HIDE_MENUBAR,
   OPTION_GEOMETRY,
@@ -211,6 +217,15 @@ struct poptOption options[] = {
     N_("Open a new tab in the last-opened window with the given profile ID. Used internally to save sessions."),
     N_("PROFILEID")
   },
+  {
+    "role",
+    '\0',
+    POPT_ARG_STRING,
+    NULL,
+    OPTION_ROLE,
+    N_("Set the role for the last-specified window; applies to only one window; can be specified once for each window you create from the command line."),
+    N_("ROLE")
+  },
   {
     "show-menubar",
     '\0',
@@ -474,6 +489,7 @@ typedef struct
   gboolean menubar_state;
 
   char *geometry;
+  char *role;
   
 } InitialWindow;
 
@@ -516,6 +532,7 @@ initial_window_new (const char *profile,
   iw->force_menubar_state = FALSE;
   iw->menubar_state = FALSE;
   iw->geometry = NULL;
+  iw->role = NULL;
   
   return iw;
 }
@@ -526,6 +543,7 @@ initial_window_free (InitialWindow *iw)
   g_list_foreach (iw->tabs, (GFunc) initial_tab_free, NULL);
   g_list_free (iw->tabs);
   g_free (iw->geometry);
+  g_free (iw->role);
   g_free (iw);
 }
 
@@ -812,6 +830,31 @@ parse_options_callback (poptContext              ctx,
       }
       break;
 
+    case OPTION_ROLE:
+      {
+        InitialWindow *iw;
+
+
+        if (arg == NULL)
+          {
+            g_printerr (_("Option --role requires an argument giving the role\n"));
+            exit (1);
+          }
+            
+        if (results->initial_windows)
+          {
+            iw = g_list_last (results->initial_windows)->data;
+            if (iw->role)
+              {
+                g_printerr (_("Two roles given for one window\n"));
+                exit (1);
+              }
+
+            iw->role = g_strdup (arg);
+          }
+      }
+      break;
+
     case OPTION_GEOMETRY:
       {
         InitialWindow *iw;
@@ -1037,7 +1080,8 @@ new_terminal_with_options (OptionParsingResults *results)
                                          it->exec_argv,
                                          iw->geometry,
                                          it->title,
-                                         it->working_dir);
+                                         it->working_dir,
+                                         iw->role);
 
               current_window = g_list_last (app->windows)->data;
             }
@@ -1050,7 +1094,8 @@ new_terminal_with_options (OptionParsingResults *results)
                                          it->exec_argv,
                                          NULL,
                                          it->title,
-                                         it->working_dir);
+                                         it->working_dir,
+                                         NULL);
             }
           
           tmp2 = tmp2->next;
@@ -1177,6 +1222,7 @@ main (int argc, char **argv)
     }  
 
   terminal_accels_init (conf);
+  terminal_encoding_init (conf);
   
   terminal_profile_initialize (conf);
   sync_profile_list (FALSE, NULL);
@@ -1191,7 +1237,6 @@ main (int argc, char **argv)
                     G_CALLBACK (client_die_cb),
                     NULL);
 
-
   if (new_terminal_with_options (results))
     return 1;
 
@@ -1235,10 +1280,12 @@ terminal_app_new_terminal (TerminalApp     *app,
                            char           **override_command,
                            const char      *geometry,
                            const char      *title,
-                           const char      *working_dir)
+                           const char      *working_dir,
+                           const char      *role)
 {
   TerminalScreen *screen;
   gboolean window_created;
+  char *new_role;
   
   g_return_if_fail (profile);
 
@@ -1254,6 +1301,21 @@ terminal_app_new_terminal (TerminalApp     *app,
                         app);
       
       app->windows = g_list_append (app->windows, window);
+
+      if (role == NULL)
+        {
+          /* Invent a unique-enough number for the role */
+          new_role = g_strdup_printf ("gnome-terminal-%d-%d-%d",
+                                      getpid (),
+                                      g_random_int (),
+                                      time (NULL));
+          gtk_window_set_role (GTK_WINDOW (window), new_role);
+          g_free (new_role);
+        }
+      else
+        {
+          gtk_window_set_role (GTK_WINDOW (window), role);
+        }
     }
 
   if (force_menubar_state)
@@ -1551,6 +1613,52 @@ terminal_app_edit_keybindings (TerminalApp     *app,
   gtk_window_present (GTK_WINDOW (app->edit_keys_dialog));
 }
 
+static void
+edit_encodings_destroyed_callback (GtkWidget   *new_profile_dialog,
+                                   TerminalApp *app)
+{
+  app->edit_encodings_dialog = NULL;
+}
+
+void
+terminal_app_edit_encodings (TerminalApp     *app,
+                             GtkWindow       *transient_parent)
+{
+  GtkWindow *old_transient_parent;
+
+  if (app->edit_encodings_dialog == NULL)
+    {      
+      old_transient_parent = NULL;      
+
+      /* passing in transient_parent here purely for the
+       * glade error dialog
+       */
+      app->edit_encodings_dialog =
+        terminal_encoding_dialog_new (transient_parent);
+
+      if (app->edit_encodings_dialog == NULL)
+        return; /* glade file missing */
+      
+      g_signal_connect (G_OBJECT (app->edit_encodings_dialog),
+                        "destroy",
+                        G_CALLBACK (edit_encodings_destroyed_callback),
+                        app);
+    }
+  else 
+    {
+      old_transient_parent = gtk_window_get_transient_for (GTK_WINDOW (app->edit_encodings_dialog));
+    }
+  
+  if (old_transient_parent != transient_parent)
+    {
+      gtk_window_set_transient_for (GTK_WINDOW (app->edit_encodings_dialog),
+                                    transient_parent);
+      gtk_widget_hide (app->edit_encodings_dialog); /* re-show the window on its new parent */
+    }
+  
+  gtk_widget_show_all (app->edit_encodings_dialog);
+  gtk_window_present (GTK_WINDOW (app->edit_encodings_dialog));
+}
 
 enum
 {
@@ -2644,6 +2752,7 @@ terminal_app_get_clone_command (TerminalApp *app,
                       */
   argc += n_windows; /* one --show-menubar or --hide-menubar per window */
 
+  argc += n_windows; /* one --role per window */
 
   argc += n_tabs - n_windows; /* one --with-tab-profile-internal-id
                                * per extra tab
@@ -2696,6 +2805,9 @@ terminal_app_get_clone_command (TerminalApp *app,
               else
                 argv[i] = g_strdup ("--hide-menubar");
               ++i;
+              argv[i] = g_strdup_printf ("--role=%s",
+                                         gtk_window_get_role (GTK_WINDOW (window)));
+              ++i;
             }
           else
             {
