@@ -46,13 +46,16 @@ void terminal_app_new_terminal (TerminalApp     *app,
                                 char           **override_command,
                                 const char      *geometry,
                                 const char      *title,
-                                const char      *working_dir);
+                                const char      *working_dir,
+                                const char      *role);
 
 void terminal_app_manage_profiles (TerminalApp     *app,
                                    GtkWindow       *transient_parent);
 
 void terminal_app_edit_keybindings (TerminalApp     *app,
                                     GtkWindow       *transient_parent);
+void terminal_app_edit_encodings   (TerminalApp     *app,
+                                    GtkWindow       *transient_parent);
 
 void terminal_util_set_labelled_by          (GtkWidget  *widget,
                                              GtkLabel   *label);
